from pydantic import BaseModel
from typing import Optional
import typing as T

class Member(BaseModel):
    user_id: int
    nickname: str
    card: T.Optional[str]
    sex: str
    age: int
    area: str
    level: str
    role: T.Optional[str]
    title: T.Optional[str]
    # 以下是 getGroupMemberInfo 返回的更多结果
    group_id: Optional[int]
    join_time: Optional[int]
    last_sent_time: Optional[int]
    unfriendly: Optional[bool]
    title_expire_time: Optional[int]
    card_changeable: Optional[bool]
    shut_up_timestamp: Optional[int]

class Anonymous(BaseModel):
    id: int
    name: str
    flag: str

class Group(BaseModel):
    group_id: int
    group_name: str
    group_memo: str
    group_create_time: int
    group_level: int
    member_count: int
    max_member_count: int

class HonorListNode(BaseModel):
    user_id: int
    nickname: str
    avatar: str
    description: Optional[str]
    day_count: Optional[int]

class Honor(BaseModel):
    group_id: int
    current_talkative: Optional[HonorListNode]
    talkative_list: Optional[T.List[HonorListNode]]
    performer_list: Optional[T.List[HonorListNode]]
    legend_list: Optional[T.List[HonorListNode]]
    strong_newbie_list: Optional[T.List[HonorListNode]]
    emotion_list: Optional[T.List[HonorListNode]]

class AtAllRemain(BaseModel):
    can_at_all: bool
    remain_at_all_count_for_group: int
    remain_at_all_count_for_uin: int


import typing as T

from .event.models import BotMessage, Message, Anonymous, ForwardMessages
from .entities import *
from .entities.components import Node
from .network import fetch


class CQHTTP_Protocol:
    baseurl_http: str
    protocol_params: dict = {}

    async def sendFriendMessage(self,
                                user_id: int,
                                message: T.Union[str, list],
                                group_id: T.Optional[int] = None,
                                auto_escape: bool = False) -> T.Union[BotMessage, bool]:
        if isinstance(message, list):
            _message = ""
            for chain in message:
                _message += chain.toString()
            message = _message
        payload = {
            "user_id": user_id,
            "message": message,
            "auto_escape": auto_escape
        }
        if group_id:
            payload["group_id"] = group_id
        result = await fetch.http_post(f"{self.baseurl_http}/send_private_msg", payload, params=self.protocol_params)
        if result["status"] == "ok":
            return BotMessage.parse_obj(result["data"])
        return False

    async def sendGroupMessage(self,
                               group_id: int,
                               message: T.Union[str, list],
                               auto_escape: bool = False) -> T.Union[BotMessage, bool]:
        if isinstance(message, list):
            _message = ""
            for chain in message:
                _message += chain.toString()
            message = _message
        result = await fetch.http_post(f"{self.baseurl_http}/send_group_msg", {
            "group_id": group_id,
            "message": message,
            "auto_escape": auto_escape
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return BotMessage.parse_obj(result["data"])
        return False

    async def sendGroupForwardMessage(self,
                                      group_id: int,
                                      messages: list) -> T.Union[BotMessage, bool]:
        for i in range(len(messages)):
            if isinstance(messages[i], Node):
                messages[i] = messages[i].toDict()
        result = await fetch.http_post(f"{self.baseurl_http}/send_group_forward_msg", {
            "group_id": group_id,
            "messages": messages
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return BotMessage.parse_obj(result["data"])
        return False

    async def recall(self, message_id: int) -> bool:
        result = await fetch.http_post(f"{self.baseurl_http}/delete_msg", {
            "message_id": message_id
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return True
        return False

    async def getMessage(self, message_id: int) -> T.Union[Message, bool]:
        result = await fetch.http_post(f"{self.baseurl_http}/get_msg", {
            "message_id": message_id
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return Message.parse_obj(result["data"])
        return False

    async def getForwardMessage(self, message_id: int) -> T.Union[ForwardMessages, bool]:
        result = await fetch.http_post(f"{self.baseurl_http}/get_forward_msg", {
            "message_id": message_id
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return ForwardMessages.parse_obj(result["data"])
        return False

    async def getImage(self, file: str) -> T.Union[ImageFile, bool]:
        result = await fetch.http_post(f"{self.baseurl_http}/get_image", {
            "file": file
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return ImageFile.parse_obj(result["data"])
        return False

    async def kick(self,
                   group_id: int,
                   user_id: int,
                   reject_add_request: bool = False) -> bool:
        result = await fetch.http_post(f"{self.baseurl_http}/set_group_kick", {
            "group_id": group_id,
            "user_id": user_id,
            "reject_add_request": reject_add_request
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return True
        return False

    async def mute(self,
                   group_id: int,
                   user_id: int,
                   duration: int = 30 * 60) -> bool:
        result = await fetch.http_post(f"{self.baseurl_http}/set_group_ban", {
            "group_id": group_id,
            "user_id": user_id,
            "duration": duration
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return True
        return False

    async def unmute(self, group_id: int, user_id: int) -> bool:
        return await self.mute(group_id, user_id, 0)

    async def muteAnonymous(self,
                            group_id: int,
                            flag: str,
                            duration: int = 30 * 60,
                            anonymous: Anonymous = None):  # TODO
        result = await fetch.http_post(f"{self.baseurl_http}/set_group_anonymous_ban", {
            "group_id": group_id,
            "flag": flag,
            "duration": duration
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return True
        return False

    async def muteAll(self,
                      group_id: int,
                      enable: bool = True) -> bool:
        result = await fetch.http_post(f"{self.baseurl_http}/set_group_whole_ban", {
            "group_id": group_id,
            "enable": enable
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return True
        return False

    async def setGroupAdmin(self,
                            group_id: int,
                            user_id: int,
                            enable: bool = True) -> bool:
        result = await fetch.http_post(f"{self.baseurl_http}/set_group_admin", {
            "group_id": group_id,
            "user_id": user_id,
            "enable": enable
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return True
        return False

    async def setGroupAnonymous(self,
                                group_id: int,
                                enable: bool = True) -> bool:  # TODO go-cqhttp 暂未支持
        result = await fetch.http_post(f"{self.baseurl_http}/set_group_anonymous", {
            "group_id": group_id,
            "enable": enable
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return True
        return False

    async def setGroupCard(self,
                           group_id: int,
                           user_id: int,
                           card: str = "") -> bool:
        result = await fetch.http_post(f"{self.baseurl_http}/set_group_card", {
            "group_id": group_id,
            "user_id": user_id,
            "card": card
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return True
        return False

    async def setGroupName(self,
                           group_id: int,
                           group_name: str) -> bool:
        result = await fetch.http_post(f"{self.baseurl_http}/set_group_name", {
            "group_id": group_id,
            "group_name": group_name
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return True
        return False

    async def leave(self,
                    group_id: int,
                    is_dismiss: bool = False) -> bool:
        result = await fetch.http_post(f"{self.baseurl_http}/set_group_leave", {
            "group_id": group_id,
            "is_dismiss": is_dismiss
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return True
        return False

    async def setGroupSpecialTitle(self,
                                   group_id: int,
                                   user_id: int,
                                   special_title: str = "",
                                   duration: int = -1) -> bool:
        result = await fetch.http_post(f"{self.baseurl_http}/set_group_special_title", {
            "group_id": group_id,
            "user_id": user_id,
            "special_title": special_title,
            "duration": duration
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return True
        return False

    async def setFriendRequest(self,
                               flag: str,
                               approve: bool = True,
                               remark: str = "") -> bool:
        result = await fetch.http_post(f"{self.baseurl_http}/set_friend_add_request", {
            "flag": flag,
            "approve": approve,
            "remark": remark
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return True
        return False

    async def setGroupRequest(self,
                              flag: str,
                              sub_type: str,
                              approve: bool = True,
                              reason: str = "") -> bool:
        if sub_type not in ["add", "invite"]:
            return False
        result = await fetch.http_post(f"{self.baseurl_http}/set_group_add_request", {
            "flag": flag,
            "sub_type": sub_type,
            "approve": approve,
            "reason": reason
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return True
        return False

    async def getLoginInfo(self) -> T.Union[Bot, bool]:
        result = await fetch.http_post(f"{self.baseurl_http}/get_login_info")
        if result["status"] == "ok":
            return Bot.parse_obj(result["data"])
        return False

    async def getQiDianAccountInfo(self) -> T.Union[QiDianAccount, bool]:
        result = await fetch.http_post(f"{self.baseurl_http}/qidian_get_account_info")
        if result["status"] == "ok":
            return QiDianAccount.parse_obj(result["data"])
        return False

    async def getStrangerInfo(self,
                              user_id: int,
                              no_cache: bool = False) -> T.Union[Stranger, bool]:
        result = await fetch.http_post(f"{self.baseurl_http}/get_stranger_info", {
            "user_id": user_id,
            "no_cache": no_cache
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return Stranger.parse_obj(result["data"])
        return False

    async def getFriendList(self) -> T.Union[List[Friend], bool]:
        result = await fetch.http_post(f"{self.baseurl_http}/get_friend_list")
        if result["status"] == "ok":
            return [Friend.parse_obj(friend_info) for friend_info in result["data"]]
        return False

    async def deleteFriend(self,
                           friend_id: int) -> bool:
        result = await fetch.http_post(f"{self.baseurl_http}/delete_friend", {
            "friend_id": friend_id
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return True
        return False

    async def getUnidirectionalFriendList(self) -> T.Union[List[Friend], bool]:
        result = await fetch.http_post(f"{self.baseurl_http}/get_unidirectional_friend_list")
        if result["status"] == "ok":
            return [Friend.parse_obj(friend_info) for friend_info in result["data"]]
        return False

    async def deleteUnidirectionalFriend(self,
                                         user_id: int) -> bool:
        result = await fetch.http_post(f"{self.baseurl_http}/delete_unidirectional_friend", {
            "user_id": user_id
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return True
        return False

    async def getGroupInfo(self,
                           group_id: int,
                           no_cache: bool = False) -> T.Union[Group, bool]:
        result = await fetch.http_post(f"{self.baseurl_http}/get_group_info", {
            "group_id": group_id,
            "no_cache": no_cache
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return Group.parse_obj(result["data"])
        return False

    async def getGroupList(self) -> T.Union[List[Group], bool]:
        result = await fetch.http_post(f"{self.baseurl_http}/get_group_list")
        if result["status"] == "ok":
            return [Group.parse_obj(group_info) for group_info in result["data"]]
        return False

    async def getGroupMemberInfo(self,
                                 group_id: int,
                                 user_id: int,
                                 no_cache: bool = False) -> T.Union[Member, bool]:
        result = await fetch.http_post(f"{self.baseurl_http}/get_group_member_info", {
            "group_id": group_id,
            "user_id": user_id,
            "no_cache": no_cache
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return Member.parse_obj(result["data"])
        return False

    async def getGroupMemberList(self,
                                 group_id: int) -> T.Union[List[Member], bool]:
        result = await fetch.http_post(f"{self.baseurl_http}/get_group_member_list", {
            "group_id": group_id
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return [Member.parse_obj(member_info) for member_info in result["data"]]
        return False

    async def getGroupHonorInfo(self,
                                group_id: int,
                                type: str) -> T.Union[Honor, bool]:
        result = await fetch.http_post(f"{self.baseurl_http}/get_group_honor_info", {
            "group_id": group_id,
            "type": type
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return Honor.parse_obj(result["data"])
        return False

    async def canSendImage(self) -> bool:
        result = await fetch.http_post(f"{self.baseurl_http}/can_send_image")
        if result["status"] == "ok":
            if result["data"]["yes"]:
                return True
        return False

    async def canSendRecord(self) -> bool:
        result = await fetch.http_post(f"{self.baseurl_http}/can_send_record")
        if result["status"] == "ok":
            if result["data"]["yes"]:
                return True
        return False

    async def getVersionInfo(self) -> T.Union[AppVersion, bool]:
        result = await fetch.http_post(f"{self.baseurl_http}/get_version_info")
        if result["status"] == "ok":
            return AppVersion.parse_obj(result["data"])
        return False

    async def restartAPI(self, delay: int = 0) -> bool:
        result = await fetch.http_post(f"{self.baseurl_http}/set_restart", {
            "delay": delay
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return True
        return False

    async def setGroupPortrait(self,
                               group_id: int,
                               file: str,
                               cache: int) -> bool:
        result = await fetch.http_post(f"{self.baseurl_http}/set_restart", {
            "group_id": group_id,
            "file": file,
            "cache": cache
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return True
        return False

    async def ocrImage(self,
                       image: str) -> T.Union[OCR, bool]:
        result = await fetch.http_post(f"{self.baseurl_http}/ocr_image", {
            "image": image
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return OCR.parse_obj(result["data"])
        return False

    async def getGroupSystemMessage(self) -> T.Union[GroupSystemMessage, bool]:
        result = await fetch.http_post(f"{self.baseurl_http}/get_group_system_msg")
        if result["status"] == "ok":
            return GroupSystemMessage.parse_obj(result["data"])
        return False

    async def uploadGroupFile(self, group_id: int) -> T.Union[GroupFileSystem, bool]:
        result = await fetch.http_post(f"{self.baseurl_http}/get_group_file_system_info", {
            "group_id": group_id
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return GroupFileSystem.parse_obj(result["data"])
        return False

    async def getGroupRootFiles(self, group_id: int) -> T.Union[GroupFileTree, bool]:
        result = await fetch.http_post(f"{self.baseurl_http}/get_group_root_files", {
            "group_id": group_id
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return GroupFileTree.parse_obj(result["data"])
        return False

    async def getGroupFilesByFolder(self,
                                    group_id: int,
                                    folder_id: str) -> T.Union[GroupFileTree, bool]:
        result = await fetch.http_post(f"{self.baseurl_http}/get_group_root_files", {
            "group_id": group_id,
            "folder_id": folder_id
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return GroupFileTree.parse_obj(result["data"])
        return False

    async def getGroupFileURL(self,
                              group_id: int,
                              file_id: str,
                              busid: int) -> T.Union[str, bool]:
        result = await fetch.http_post(f"{self.baseurl_http}/get_group_file_url", {
            "group_id": group_id,
            "file_id": file_id,
            "busid": busid
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return result["data"]["url"]
        return False

    async def getStatus(self) -> T.Union[AppStatus, bool]:
        result = await fetch.http_post(f"{self.baseurl_http}/get_status")
        if result["status"] == "ok":
            return AppStatus.parse_obj(result["data"])
        return False

    async def getGroupAtAllRemain(self, group_id: int) -> T.Union[AtAllRemain, bool]:
        result = await fetch.http_post(f"{self.baseurl_http}/get_group_at_all_remain", {
            "group_id": group_id
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return AtAllRemain.parse_obj(result["data"])
        return False

    async def getVipInfo(self, user_id: int) -> T.Union[VipInfo, bool]:
        result = await fetch.http_post(f"{self.baseurl_http}/_get_vip_info", {
            "user_id": user_id
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return VipInfo.parse_obj(result["data"])
        return False

    async def sendGroupNotice(self, group_id: int, content: str):
        result = await fetch.http_post(f"{self.baseurl_http}/_send_group_notice", {
            "group_id": group_id,
            "content": content
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return True
        return False

    async def reloadEventFilter(self, file: str):
        result = await fetch.http_post(f"{self.baseurl_http}/reload_event_filter", {
            "file": file
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return True
        return False

    async def downloadFile(self, url: str, headers: str, thread_count=1):
        result = await fetch.http_post(f"{self.baseurl_http}/download_file", {
            "url": url,
            "headers": headers,
            "thread_count": thread_count
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return True
        return False

    async def getOnlineClients(self, no_cashe: bool) -> T.Union[List[Device], bool]:
        result = await fetch.http_post(f"{self.baseurl_http}/get_online_clients", {
            "no_cache": no_cashe
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return [Device.parse_obj(device) for device in result["data"]["clients"]]
        return False

    async def getGroupMessageHistory(self, group_id: int, message_seq: Optional[int] = None) -> T.Union[List[Message], bool]:
        result = await fetch.http_post(f"{self.baseurl_http}/get_group_msg_history", {
            "message_seq": message_seq,
            "group_id": group_id
        }, params=self.protocol_params)
        if result["status"] == "ok":
            print(result)
            return [Message.parse_obj(message) for message in result["data"]["messages"]]
        return False

    async def setEssenceMessage(self, message_id: int) -> bool:
        result = await fetch.http_post(f"{self.baseurl_http}/set_essence_msg", {
            "message_id": message_id
        }, params=self.protocol_params)  # 草,为什么只有手机看得到
        if result["status"] == "ok":
            return True
        return False

    async def deleteEssenceMessage(self, message_id: int) -> bool:
        result = await fetch.http_post(f"{self.baseurl_http}/delete_essence_msg", {
            "message_id": message_id
        }, params=self.protocol_params)  # 这个我没测试过
        if result["status"] == "ok":
            return True
        return False

    async def getEssenceMessageList(self, group_id: int) -> T.Union[EssenceMessage, bool]:
        result = await fetch.http_post(f"{self.baseurl_http}/get_essence_msg_list", {
            "group_id": group_id
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return EssenceMessage.parse_obj(result["data"])
        return False

    async def checkURLSafety(self, url: str) -> int:
        result = await fetch.http_post(f"{self.baseurl_http}/check_url_safely", {
            "url": url
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return result["level"]
        return False

    async def getModelShow(self, model: str) -> T.Union[List[ModelShow], bool]:
        result = await fetch.http_post(f"{self.baseurl_http}/_get_model_show", {
            "model": model
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return [ModelShow.parse_obj(_model) for _model in result["data"]["variants"]]
        return False

    async def setModelShow(self, model: str, model_show: str) -> bool:
        result = await fetch.http_post(f"{self.baseurl_http}/_set_model_show", {
            "model": model,
            "model_show": model_show
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return True
        return False

    async def getGuildServiceProfile(self) -> T.Union[BotGuild, bool]:
        '''
        获取频道系统内BOT的资料
        '''
        result = await fetch.http_post(f"{self.baseurl_http}/get_guild_service_profile")
        if result["status"] == "ok":
            return BotGuild.parse_obj(result["data"])
        return False

    async def getGuildList(self) -> T.Union[list, bool]:
        '''
        获取频道列表
        '''
        result = await fetch.http_post(f"{self.baseurl_http}/get_guild_list")
        if result["status"] == "ok":
            return [Guild.parse_obj(_guild) for _guild in result["data"]]
        return False

    async def getGuildMetaByGuest(self, guild_id: int) -> T.Union[Guild, bool]:
        '''
        通过访客获取频道元数据
        '''
        result = await fetch.http_post(f"{self.baseurl_http}/get_guild_meta_by_guest", {
            "guild_id": guild_id
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return Guild.parse_obj(result["data"])
        return False

    async def getGuildChannelList(self, guild_id: int, no_cache: bool = False) -> T.Union[list, bool]:
        '''
        获取子频道列表
        '''
        result = await fetch.http_post(f"{self.baseurl_http}/get_guild_channel_list", {
            "guild_id": guild_id,
            "no_cache": no_cache
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return [Channel.parse_obj(_channel) for _channel in result["data"]]
        return False

    async def getGuildMembers(self, guild_id: int) -> T.Union[GuildMembers, bool]:
        '''
        获取频道成员列表
        '''
        result = await fetch.http_post(f"{self.baseurl_http}/get_guild_members", {
            "guild_id": guild_id
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return GuildMembers.parse_obj(result["data"])
        return False

    async def sendGuildChannelMessage(self, guild_id: int, channel_id: int, message: T.Union[str, list]) -> T.Union[BotMessage, bool]:
        '''
        发送信息到子频道
        '''
        if isinstance(message, list):
            _message = ""
            for chain in message:
                _message += chain.toString()
            message = _message
        result = await fetch.http_post(f"{self.baseurl_http}/send_guild_channel_msg", {
            "guild_id": guild_id,
            "channel_id": channel_id,
            "message": message
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return BotMessage.parse_obj(result["data"])
        return False

    async def getGuildRoles(self, guild_id: int) -> T.Union[Role, bool]:
        '''
        获取频道身份组列表
        '''
        result = await fetch.http_post(f"{self.baseurl_http}/get_guild_roles", {
            "guild_id": guild_id
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return [Role.parse_obj(_role) for _role in result["data"]]
        return False

    async def createGuildRole(self, guild_id: int, name: str, color: int, independent: bool, initial_users: T.List[int] = []) -> T.Union[Role, bool]:
        '''
        创建频道身份组
        '''
        result = await fetch.http_post(f"{self.baseurl_http}/create_guild_role", {
            "guild_id": guild_id,
            "name": name,
            "color": color,
            "independent": independent,
            "initial_users": initial_users
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return Role.parse_obj(result["data"])
        return False

    async def deleteGuildRole(self, guild_id: int, role_id: int) -> bool:
        '''
        删除频道身份组
        '''
        result = await fetch.http_post(f"{self.baseurl_http}/delete_guild_role", {
            "guild_id": guild_id,
            "role_id": role_id
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return True
        return False

    async def setGuildMemberRole(self, guild_id: int, role_id: int, users: T.List[int]) -> bool:
        '''
        设置用户在频道中的身份组
        '''
        result = await fetch.http_post(f"{self.baseurl_http}/set_guild_member_role", {
            "guild_id": guild_id,
            "role_id": role_id,
            "users": users
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return True
        return False

    async def editGuildRole(self, guild_id: int, role_id: int, name: str, color: int, independent: bool) -> bool:
        '''
        设置用户在频道中的身份组
        '''
        result = await fetch.http_post(f"{self.baseurl_http}/update_guild_role", {
            "guild_id": guild_id,
            "role_id": role_id,
            "name": name,
            "color": color,
            "independent": independent # TODO gocq 中参数名写错了，待后续确认
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return True
        return False

    async def getTopicChannelFeeds(self, guild_id: int, channel_id: int) -> T.Union[T.List[TopicChannelFeed], bool]:
        '''
        获取论坛子频道帖子列表
        '''
        result = await fetch.http_post(f"{self.baseurl_http}/get_topic_channel_feeds", {
            "guild_id": guild_id,
            "channel_id": channel_id
        }, params=self.protocol_params)
        if result["status"] == "ok":
            return [TopicChannelFeed.parse_obj(_feed) for _feed in result["data"]]
        return False


from pydantic import BaseModel
import typing as T

class Depend:
    def __init__(self, func, middlewares=[], cache=True):
        self.func = func
        self.middlewares = middlewares
        self.cache = cache

class ExecutorProtocol(BaseModel):
    callable: T.Callable
    dependencies: T.List[Depend]
    middlewares: T.List

    class Config:
        arbitrary_types_allowed = True

from . import InternalEvent
from pydantic import BaseModel

class UnexpectedException(BaseModel):
    error: Exception
    event: InternalEvent

    class Config:
        arbitrary_types_allowed = True


import sys
import aiohttp
import asyncio
import inspect
import copy
import pydantic
import traceback

from contextlib import AsyncExitStack
from typing import Callable, NamedTuple, Awaitable, Any, List, Dict
from functools import partial
from async_lru import alru_cache

from .event import InternalEvent, ExternalEvent
from .event.models import MessageTypes, NoticeTypes, RequestTypes, Friend, Member, GuildMember
from .event.builtins import ExecutorProtocol, Depend
from .event.models import (
    FriendMessage, GroupMessage, GuildMessage, MessageItemType
)
from .event.enums import ExternalEvents
from .misc import argument_signature, raiser, TRACEBACKED
from .protocol import CQHTTP_Protocol
from .logger import logger

class CQHTTP(CQHTTP_Protocol):
    event: Dict[
        str, List[Callable[[Any], Awaitable]]
    ] = {}
    lifecycle: Dict[str, List[Callable]] = {
        "start": [],
        "end": [],
        "around": []
    }
    global_dependencies: List[Depend]
    global_middlewares: List

    def __init__(self,
                 host: str = None,
                 port: int = None,
                 http_port: int = None,
                 token: str = None,
                 global_dependencies: List[Depend] = None,
                 global_middlewares: List = None):
        self.global_dependencies = global_dependencies or []
        self.global_middlewares = global_middlewares or []

        self.headers = {}
        if token is not None:
            self.headers["Authorization"] = f"Bearer {token}"
            self.protocol_params = {
                "access_token": token
            }
        self.baseurl = f"http://{host}:{port}"
        self.baseurl_http = f"http://{host}:{http_port}"

    async def ws_event(self):
        async with aiohttp.ClientSession() as session:
            async with session.ws_connect(f"{self.baseurl}", headers=self.headers) as ws_connection:
                logger.info("Protocol: connected")
                while True:
                    try:
                        received_data = await ws_connection.receive_json()
                    except TypeError:
                        continue
                    if received_data:
                        post_type = received_data["post_type"]
                        try:
                            if post_type == "message":
                                received_data = MessageTypes[received_data["message_type"]].parse_obj(received_data)
                            elif post_type == "notice":
                                received_data = NoticeTypes[received_data["notice_type"]].parse_obj(received_data)
                            elif post_type == "request":
                                received_data = RequestTypes[received_data["request_type"]].parse_obj(received_data)
                            else:
                                continue
                        except KeyError:
                            logger.error("Protocol: data parse error: " + received_data)
                            continue
                        except pydantic.error_wrappers.ValidationError:
                            logger.error("Protocol: data parse error: " + received_data)
                            continue
                        await self.queue.put(InternalEvent(
                            name=self.getEventCurrentName(type(received_data)),
                            body=received_data
                        ))

    async def event_runner(self):
        while True:
            try:
                event_context: NamedTuple[InternalEvent] = await asyncio.wait_for(self.queue.get(), 3)
            except asyncio.TimeoutError:
                continue

            if event_context.name in self.registeredEventNames:
                logger.info(f"Event: handling a event: {event_context.name}")
                for event_body in list(self.event.values()) \
                        [self.registeredEventNames.index(event_context.name)]:
                    if event_body:
                        running_loop = asyncio.get_running_loop()
                        running_loop.create_task(self.executor(event_body, event_context))

    @property
    def registeredEventNames(self):
        return [self.getEventCurrentName(i) for i in self.event.keys()]

    async def executor(self,
                       executor_protocol: ExecutorProtocol,
                       event_context,
                       extra_parameter={},
                       lru_cache_sets=None
                       ):
        lru_cache_sets = lru_cache_sets or {}
        executor_protocol: ExecutorProtocol
        for depend in executor_protocol.dependencies:
            if not inspect.isclass(depend.func):
                depend_func = depend.func
            elif hasattr(depend.func, "__call__"):
                depend_func = depend.func.__call__
            else:
                raise TypeError("must be callable.")

            if depend_func in lru_cache_sets and depend.cache:
                depend_func = lru_cache_sets[depend_func]
            else:
                if depend.cache:
                    original = depend_func
                    if inspect.iscoroutinefunction(depend_func):
                        depend_func = alru_cache(depend_func)
                    else:
                        depend_func = lru_cache(depend_func)
                    lru_cache_sets[original] = depend_func

            result = await self.executor_with_middlewares(
                depend_func, depend.middlewares, event_context, lru_cache_sets
            )
            if result is TRACEBACKED:
                return TRACEBACKED

        ParamSignatures = argument_signature(executor_protocol.callable)
        PlaceAnnotation = self.get_annotations_mapping()
        CallParams = {}
        for name, annotation, default in ParamSignatures:
            if default:
                if isinstance(default, Depend):
                    if not inspect.isclass(default.func):
                        depend_func = default.func
                    elif hasattr(default.func, "__call__"):
                        depend_func = default.func.__call__
                    else:
                        raise TypeError("must be callable.")

                    if depend_func in lru_cache_sets and default.cache:
                        depend_func = lru_cache_sets[depend_func]
                    else:
                        if default.cache:
                            original = depend_func
                            if inspect.iscoroutinefunction(depend_func):
                                depend_func = alru_cache(depend_func)
                            else:
                                depend_func = lru_cache(depend_func)
                            lru_cache_sets[original] = depend_func

                    CallParams[name] = await self.executor_with_middlewares(
                        depend_func, default.middlewares, event_context, lru_cache_sets
                    )
                    continue
                else:
                    raise RuntimeError("checked a unexpected default value.")
            else:
                if annotation in PlaceAnnotation:
                    CallParams[name] = PlaceAnnotation[annotation](event_context)
                    continue
                else:
                    if name not in extra_parameter:
                        raise RuntimeError(f"checked a unexpected annotation: {annotation}")

        async with AsyncExitStack() as stack:
            sorted_middlewares = self.sort_middlewares(executor_protocol.middlewares)
            for async_middleware in sorted_middlewares['async']:
                await stack.enter_async_context(async_middleware)
            for normal_middleware in sorted_middlewares['normal']:
                stack.enter_context(normal_middleware)

            return await self.run_func(executor_protocol.callable, **CallParams, **extra_parameter)
    
    async def _run(self):
        loop = asyncio.get_event_loop()
        self.queue = asyncio.Queue(loop=loop) if sys.version_info.minor < 10 else asyncio.Queue()
        loop.create_task(self.ws_event())
        loop.create_task(self.event_runner())

        await self.queue.put(InternalEvent(
            name=self.getEventCurrentName("AppInitEvent"),
            body={}
        ))
        try:
            for start_callable in self.lifecycle['start']:
                await self.run_func(start_callable, self)

            for around_callable in self.lifecycle['around']:
                await self.run_func(around_callable, self)

        except KeyboardInterrupt:
            logger.info("catched Ctrl-C, exiting..")
        except Exception as e:
            traceback.print_exc()
        finally:
            for around_callable in self.lifecycle['around']:
                await self.run_func(around_callable, self)

            for end_callable in self.lifecycle['end']:
                await self.run_func(end_callable, self)
    
    def run(self):
        loop = asyncio.get_event_loop()
        loop.run_until_complete(self._run())
        loop.run_forever()

    def receiver(self,
                 event_name,
                 dependencies: List[Depend] = None,
                 use_middlewares: List[Callable] = None):
        def receiver_warpper(func: Callable):
            if not inspect.iscoroutinefunction(func):
                raise TypeError("event body must be a coroutine function.")
            
            self.event.setdefault(event_name, [])
            self.event[event_name].append(ExecutorProtocol(
                callable=func,
                dependencies=(dependencies or []) + self.global_dependencies,
                middlewares=(use_middlewares or []) + self.global_middlewares
            ))
            return func

        return receiver_warpper

    def getEventCurrentName(self, event_value):
        class_list = (
            GroupMessage,
            FriendMessage,
            GuildMessage,
            *self.get_event_class_name()
        )
        if inspect.isclass(event_value) and issubclass(event_value, ExternalEvent):  # subclass
            return event_value.__name__
        elif isinstance(event_value, class_list):  # normal class
            return event_value.__class__.__name__
        elif event_value in class_list:  # message
            return event_value.__name__
        elif isinstance(event_value, (  # enum
            MessageItemType,
            ExternalEvents
        )):
            return event_value.name
        else:
            return event_value

    def get_annotations_mapping(self):
        return {
            CQHTTP: lambda k: self,
            FriendMessage: lambda k: k.body \
                if self.getEventCurrentName(k.body) == "FriendMessage" else \
                raiser(ValueError("you cannot setting a unbind argument.")),
            GroupMessage: lambda k: k.body \
                if self.getEventCurrentName(k.body) == "GroupMessage" else \
                raiser(ValueError("you cannot setting a unbind argument.")),
            GuildMessage: lambda k: k.body \
                if self.getEventCurrentName(k.body) == "GuildMessage" else \
                raiser(ValueError("you cannot setting a unbind argument.")),
            Friend: lambda k: k.body.sender \
                if self.getEventCurrentName(k.body) == "FriendMessage" else \
                raiser(ValueError("Friend is not enable in this type of event.")),
            Member: lambda k: k.body.sender \
                if self.getEventCurrentName(k.body) == "GroupMessage" else \
                raiser(ValueError("Group is not enable in this type of event.")),
            GuildMember: lambda k: k.body.sender \
                if self.getEventCurrentName(k.body) == "GuildMessage" else \
                raiser(ValueError("Group is not enable in this type of event.")),
            "Sender": lambda k: k.body.sender \
                if self.getEventCurrentName(k.body) in MessageTypes else \
                raiser(ValueError("Sender is not enable in this type of event.")),
            "Type": lambda k: self.getEventCurrentName(k.body),
            **self.gen_event_anno()
        }

    def get_event_class_name(self):
        def warpper(name, event_context):
            if name != event_context.name:
                raise ValueError("cannot look up a non-listened event.")
            return event_context.body
        
        return {
            event_class.value for event_name, event_class in ExternalEvents.__members__.items()
        }

    def gen_event_anno(self):
        def warpper(name, event_context):
            if name != event_context.name:
                raise ValueError("cannot look up a non-listened event.")
            return event_context.body
        
        return {
            event_class.value: partial(warpper, copy.copy(event_name)) \
            for event_name, event_class in ExternalEvents.__members__.items()
        }

    @staticmethod
    def sort_middlewares(iterator):
        return {
            "async": [
                i for i in iterator if all([
                    hasattr(i, "__aenter__"),
                    hasattr(i, "__aexit__")
                ])
            ],
            "normal": [
                i for i in iterator if all([
                    hasattr(i, "__enter__"),
                    hasattr(i, "__exit__")
                ])
            ]
        }

    @staticmethod
    async def run_func(func, *args, **kwargs):
        if inspect.iscoroutinefunction(func):
            await func(*args, **kwargs)
        else:
            func(*args, **kwargs)



import json
import mimetypes
import typing as T
from pathlib import Path
from .logger import logger

import aiohttp

class fetch:
    @staticmethod
    async def http_post(url, data_map=None, **kwargs):
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=data_map, **kwargs) as response:
                data = await response.text(encoding="utf-8")
                logger.debug(f"Network: requested url={url}, by data_map={data_map}, and status={response.status}, data={data}")
                response.raise_for_status()
        try:
            return json.loads(data)
        except json.decoder.JSONDecodeError:
            logger.error(f"Network: requested {url} with {data_map}, responsed {data}, decode failed...")

    @staticmethod
    async def http_get(url, params=None, **kwargs): 
        async with aiohttp.ClientSession() as session:
            async with session.get(url, params=params, **kwargs) as response:
                response.raise_for_status()
                data = await response.text(encoding="utf-8")
                logger.debug(f"Network: requested url={url}, by params={params}, and status={response.status}, data={data}")
        try:
            return json.loads(data)
        except json.decoder.JSONDecodeError:
            logger.error(f"Network: requested {url} with {params}, responsed {data}, decode failed...")



from pydantic import BaseModel
import typing as T

class File(BaseModel):
    id: str
    name: str
    size: int
    busid: int

class OfflineFile(BaseModel):
    name: str
    size: int
    url: str

class ImageFile(BaseModel):
    size: int
    filename: str
    url: str

class GroupFileSystem(BaseModel):
    file_count: int
    limit_count: int
    used_space: int
    total_space: int

class GroupFile(BaseModel):
    file_id: str
    file_name: str
    busid: int
    file_size: int
    upload_time: int
    dead_time: int
    modify_time: int
    download_times: int
    uploader: int
    uploader_name: str

class GroupFolder(BaseModel):
    folder_id: str
    folder_name: str
    create_time: int
    creator: int
    creator_name: str
    total_file_count: int

class GroupFileTree(BaseModel):
    files: T.List[GroupFile]
    folders: T.List[GroupFolder]


from pydantic import BaseModel, Field
from typing import Optional, List

class Bot(BaseModel):
    user_id: int
    nickname: str

class QiDianAccount(BaseModel):
    master_id: int
    ext_name: str
    create_time: int

class Stranger(BaseModel):
    user_id: int
    nickname: str
    sex: str
    age: int
    qid: str

class AppVersion(BaseModel):
    app_name: str
    app_version: str
    app_full_name: str
    protocol_version: str
    coolq_edition: str
    coolq_directory: str
    go_cqhttp: bool = Field(..., alias="go-cqhttp")
    plugin_version: str
    plugin_build_number: int
    plugin_build_configuration: str
    runtime_version: str
    runtime_os: str
    version: str
    protocol: int

class NetworkStatistics(BaseModel):
    packet_received: int
    packet_sent: int
    packet_lost: int
    message_received: int
    message_sent: int
    disconnect_times: int
    lost_times: int

class AppStatus(BaseModel):
    app_initialized: bool
    app_enabled: bool
    plugins_good: bool
    app_good: bool
    online: bool
    good: bool
    stat: NetworkStatistics

class TextDetection(BaseModel):
    text: str
    confidence: int
    coordinates: str

class OCR(BaseModel):
    texts: TextDetection
    language: str

class InvitedRequest(BaseModel):
    request_id: int
    invitor_uin: int
    invitor_nick: str
    group_id: int
    group_name: str
    checked: bool
    actor: int

class JoinRequest(BaseModel):
    request_id: int
    requester_uin: int
    requester_nick: str
    message: str
    group_id: int
    group_name: str
    checked: bool
    actor: int

class GroupSystemMessage(BaseModel):
    invited_requests: Optional[InvitedRequest]
    join_requests: Optional[JoinRequest]

class VipInfo(BaseModel):
    user_id: int
    nickname: str
    level: int
    level_speed: float
    vip_level: str
    vip_growth_speed: int
    vip_growth_total: int

class EssenceMessage(BaseModel):
    sender_id: int
    sender_nick: int
    sender_time: int
    operator_id: int
    operator_nick: int
    operator_time: int
    message_id: int

class ModelShow(BaseModel):
    model_show: str
    need_pay: bool



from setuptools import setup, find_packages

setup(
    name="nakuru-project",
    version="1.0.0",
    author="Lxns-Network",
    author_email="joinchang1206@gmail.com",
    description="一款为 go-cqhttp 的正向 WebSocket 设计的 Python SDK，支持纯 CQ 码与消息链的转换处理",
    url="https://github.com/Lxns-Network/nakuru-project",
    packages=find_packages(include=("nakuru", "nakuru.*")),
    install_requires=[
        "aiohttp",
        "pydantic",
        "Logbook",
        "async_lru"
    ]
)


from nakuru import (
    CQHTTP,
    GroupMessage
)
from nakuru.entities.components import Plain, Node, Image

app = CQHTTP(
    host="127.0.0.1",
    port=6700,
    http_port=5700
)

@app.receiver("GroupMessage")
async def _(app: CQHTTP, source: GroupMessage):
    # 方法 1
    await app.sendGroupForwardMessage(source.group_id, [
        Node(name="落雪ちゃん", uin=2941383730, content=[
            Plain(text="nc什么时候cos小老师")
        ]),
        Node(name="盐焗雾喵", uin=2190945952, content=[
            Plain(text="今晚就cos小老师")
        ]),
        Node(name="Rosemoe♪ ~ requiem ~", uin=2073412493, content=[
            Plain(text="好耶"),
            Image.fromFileSystem("./src/1.jpg")
        ])
    ])
    # 方法 2
    await app.sendGroupForwardMessage(source.group_id, [
        {
            "type": "node",
            "data": {
                "name": "落雪ちゃん",
                "uin": 2941383730,
                "content": "nc什么时候cos小老师"
            }
        },
        {
            "type": "node",
            "data": {
                "name": "盐焗雾喵",
                "uin": 2190945952,
                "content": "今晚就cos小老师"
            }
        },
        {
            "type": "node",
            "data": {
                "name": "Rosemoe♪ ~ requiem ~",
                "uin": 2073412493,
                "content": [
                    {
                        "type": "text",
                        "data": {"text": "好耶"}
                    },
                    {
                        "type": "image",
                        "data": {"file": "file:///D:/src/1.jpg"}  # 此处需要绝对路径
                    }
                ]
            }
        }
    ])

app.run()
